/***************************************************************************
 *   Copyright (C) 2006 by Michael Schulze                                 *
 *   mike.s@genion.de                                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef CONSISTENCYCHECK_H
#define CONSISTENCYCHECK_H

#include "ipodutility.h"
#include <qdict.h>

/**
 * Does some consistency checks between the database and the filesystem
 * @author Michael Schulze <mike.s@genion.de>
 */
class ConsistencyCheck : public IPodUtility
{

    typedef QDict<itunesdb::TrackPtrList> MultipleTrackDict;
    typedef QDictIterator<itunesdb::TrackPtrList> MultipleTrackDictIter;

    void printMissingFilesTable( const ITunesDB& itunesdb, QTextStream& stream );
    void printMultipleReferencedFiles( const ITunesDB& itunesdb, QTextStream& stream );
    void checkForUnreferencedFiles( KPod& ipod, QTextStream& stream );
    void removeMissingFileTracks( KPod& ipod );

public:
    ConsistencyCheck();
    virtual ~ConsistencyCheck();

    virtual const QString& getName();
    virtual const QString& getDefaultMimeType();
    virtual void handleRequest( KPod& ipod, const KURL& url, QByteArray& response, QString& mimeType );

protected:
    static const QString defaultmimetype;
    static const QString name;
};

#endif
