/***************************************************************************
 *   Copyright (C) 2004 by Michael Schulze                                 *
 *   mike.s@genion.de                                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "fileutils.h"

#include <id3/tag.h>
#include <id3/misc_support.h>

#include <qfileinfo.h>

#include <kdebug.h>

FileUtils::FileUtils()
{
}


FileUtils::~FileUtils()
{
}


class MP3MetaDataProvider {
    QFileInfo fileInfo;
    ID3_Tag id3tag;
    const Mp3_Headerinfo * mp3header;

    inline Q_UINT32 getUIntField( ID3_FrameID frameID, ID3_FieldID fieldID ) {
        Q_UINT32 result = 0;
        ID3_Frame * frame = id3tag.Find( frameID );
        if ( frame != NULL ) {
            ID3_Field * field = frame->GetField( fieldID );
            if ( field != NULL ) {
                result = field->Get();
            }
        }
        return result;
    }

public:
    MP3MetaDataProvider( const QString& filename ) : fileInfo( filename ), id3tag( filename ) {
        // kdDebug() << "MP3MetaDataProvider( const QString& filename )" << endl;
        mp3header = id3tag.GetMp3HeaderInfo();
        // kdDebug() << "MP3MetaDataProvider( const QString& filename ) end" << endl;
    }
    virtual ~MP3MetaDataProvider() { }
    bool checkInfoSufficient() {
        // kdDebug() << "checkInfoSufficient" << endl;
        return id3tag.NumFrames() > 0 && !getArtist().isEmpty() && !getTitle().isEmpty();
    }
    // Album specific stuff
    QString getArtist() {
        // kdDebug() << "getArtist" << endl;
        return QString( ID3_GetArtist( &id3tag ) );
    }
    QString getArtistNameForSort() {
        return QString::null;
    }
    QString getAlbum() {
        // kdDebug() << "getAlbum" << endl;
        return QString( ID3_GetAlbum( &id3tag ) );
    }
    QString getAlbumForSort() {
        return QString::null;
    }
    Q_UINT16 getBPM() {
        // kdDebug() << "getBPM" << endl;
        ID3_Frame * frame = id3tag.Find( ID3FID_BPM );
        if ( frame != NULL ) {
            // kdDebug() << "end" << endl;
            return QString( ID3_GetString( frame, ID3FN_TEXT ) ).toUInt();
        }
        // kdDebug() << "end" << endl;
        return 0;
    }
    QString getCategory() {
        // kdDebug() << "getCategory" << endl;
        return QString::null;
    }
    QString getDescription() {
        // kdDebug() << "getDescription" << endl;
        return QString::null;
    }
    QString getEqualizerSetting() {
        // kdDebug() << "getEqualizerSetting" << endl;
        return QString::null;
    }
    QString getFileFormatDesc() {
        // kdDebug() << "getFileFormatDesc" << endl;
        return QString( "MPEG audio file" );
    }
    QString getTitle() {
        // kdDebug() << "getTitle" << endl;
        return QString( ID3_GetTitle( &id3tag ) );
    }
    QString getTitleForSort() {
        return QString::null;
    }
    QString getGenre() {
        // kdDebug() << "getGenre" << endl;
        int genrenum = ID3_GetGenreNum(&id3tag);
        if ( genrenum != 0xFF ) {
            // kdDebug() << "end" << endl;
            return QString( ID3_V1GENRE2DESCRIPTION( genrenum ) );
        } else {
            // kdDebug() << "end" << endl;
            return  QString( ID3_GetGenre( &id3tag ) );
        }
    }
    QString getGrouping() {
        // kdDebug() << "getGrouping" << endl;
        return QString::null;
    }
    Q_UINT32 getStartTime() {
        // kdDebug() << "getStartTime" << endl;
        return 0;
    }
    Q_UINT32 getStopTime() {
        // kdDebug() << "getStopTime" << endl;
        return 0;
    }
    Q_UINT32 getSoundCheck() {
        // kdDebug() << "getSoundCheck" << endl;
        return 0;
    }
    QString getComment() {
        // kdDebug() << "getComment" << endl;
        return QString( ID3_GetComment( &id3tag ) );
    }
    QString getTvShow() {
        // FIXME implement this (?)
        return QString::null;
    }
    QString getComposer() {
        // kdDebug() << "getComposer" << endl;
        ID3_Frame * frame = id3tag.Find( ID3FID_COMPOSER );
        if ( frame != NULL ) {
            // kdDebug() << "end" << endl;
            return QString( ID3_GetString( frame, ID3FN_TEXT ) );
        }
        // kdDebug() << "end" << endl;
        return QString::null;
    }
    QString getAlbumArtist() {
        // FIXME implement this
        return QString::null;
    }
    QString getAlbumArtistForSort() {
        return QString::null;
    }
    QString getKeywords() {
        // FIXME implement this
        return QString::null;
    }
    Q_UINT32 getSeasonNumber() {
        // FIXME implement this (?)
        return 0;
    }
    Q_UINT32 getEpisodeNumber() {
        // FIXME implement this (?)
        return 0;
    }
    Q_UINT32 getYear() {
        // kdDebug() << "getYear" << endl;
        char * yearstring = ID3_GetYear(&id3tag);
        if ( yearstring != NULL ) {
            // kdDebug() << "end" << endl;
            return atoi( yearstring );
        } else {
            // kdDebug() << "end" << endl;
            return 0;
        }
    }
    Q_UINT32 getTrackNumber() {
        // kdDebug() << "getTrackNumber" << endl;
        return ID3_GetTrackNum( &id3tag );
    }
    Q_UINT32 getTrackCount() {
        // kdDebug() << "getTrackCount" << endl;
        return 0;
    }
    Q_UINT32 getCdNumber() {
        // kdDebug() << "getCdNumber" << endl;
        return 0;
    }
    Q_UINT32 getCdCount() {
        // kdDebug() << "getCdCount" << endl;
        return 0;
    }
    // Track related technical stuff
    Q_UINT32 getSamplerate() {
        // kdDebug() << "getSamplerate" << endl;
        return 0xAC440000; // TODO read samplerate
    }
    Q_UINT32 getVolumeAdjust() {
        kdDebug() << "getVolumeAdjust" << endl;
        return getUIntField( ID3FID_VOLUMEADJ, ID3FN_VOLUMEADJ );
    }
    Q_UINT32 getPlayCount() {
        // kdDebug() << "getPlayCount" << endl;
        return getUIntField( ID3FID_PLAYCOUNTER, ID3FN_COUNTER );
    }
    Q_UINT32 getLastPlayed() {
        // kdDebug() << "getLastPlayed" << endl;
        return 0;
    }
    Q_UINT32 getTrackLength() {
        // kdDebug() << "getTrackLength" << endl;
        if ( mp3header != NULL ) {
            // kdDebug() << "end" << endl;
            return mp3header->time * 1000;    // time in milliseconds
        }
        // kdDebug() << "end" << endl;
        return 0;
    }
    Q_UINT32 getBitrate() {
        // kdDebug() << "getBitrate" << endl;
        if ( mp3header != NULL ) {
            if ( mp3header->vbr_bitrate ) {
                // kdDebug() << "end" << endl;
                return mp3header->vbr_bitrate / 1000;
            } else {
                // kdDebug() << "end" << endl;
                return mp3header->bitrate / 1000;    // Bitrate in KBps
            }
        }
        // kdDebug() << "end" << endl;
	return 0;
    }
    Q_UINT32 getFileFormatCode() {
        // kdDebug() << "getFileFormatCode" << endl;
        return 0xC;
    }
    unsigned char getRating() {
        // kdDebug() << "getRating" << endl;
        return 0;
    }
    unsigned char getVbrFlag() {
        // kdDebug() << "getVbrFlag" << endl;
        if ( mp3header != NULL ) {
            // kdDebug() << "end" << endl;
            return mp3header->vbr_bitrate ? 1 : 0;
        }
        // kdDebug() << "end" << endl;
        return 0;
    }
    itunesdb::Track::MediaType getMediaType() {   // mp3=1, aac+audible=0
        // kdDebug() << "getMediaType" << endl;
        return itunesdb::Track::Audio;
    }
    unsigned char getCompilationFlag() {
        // kdDebug() << "getCompilationFlag" << endl;
        return 0;
    }

    // Resource related technical stuff
    Q_UINT32 getFileSize() {
        // kdDebug() << "getFileSize" << endl;
        return fileInfo.size();
    }

    Q_UINT32 getLastModified() {
        // kdDebug() << "getLastModified" << endl;
        return fileInfo.lastModified().toTime_t();
    }
    Q_UINT32 getLastSkippedTime() {
        return 0;
    }
    Q_UINT32 getSkipCount() {
        return 0;
    }
    Q_UINT32 getNumPregapSamples() {
        return 0;
    }
    Q_UINT64 getNumSongSamples() {
        return 0;
    }
    Q_UINT32 getNumPostgapSamples() {
        return 0;
    }
    Q_UINT32 getGaplessDataSize() {
        return 0;
    }
    bool hasLyrics() {
        return id3tag.Find( ID3FID_SYNCEDLYRICS ) != NULL || id3tag.Find( ID3FID_UNSYNCEDLYRICS ) != NULL;
    }
};


/*!
\fn FileUtils::readMP3Metadata( const QString& filename, itunesdb::Track& track2fill )
 */
bool FileUtils::readMP3Metadata( const QString& filename, ITunesDBTrack * track2fill ) {
    QFileInfo fileinfo( filename );
    if( !fileinfo.exists() ) {
        return false;    // whatever happened here
    }

    MP3MetaDataProvider provider( filename );
    if ( !provider.checkInfoSufficient() ) {
        return false;
    }

    track2fill->readFrom( provider );

    return true;
}


QString FileUtils::getFileExtension( const QString& filename ) {
    if ( !filename.isEmpty() ) {
        return filename.section('.', -1, -1, QString::SectionIncludeLeadingSep);
    }
    return filename;
}

